import dataclasses
from typing import Dict, List

try:
    # import boto3
    import botocore
    from boto3.session import Session
except ImportError:
    print('boto3がローカル環境にインストールされていません')
    exit(1)


# 取得する認証情報の期限は12時間に設定
DURATION_SECONDS = 43200


@dataclasses.dataclass
class AwsCredential:
    """認証情報クラス"""

    __profile_name: str
    __aws_access_key_id: str
    __aws_secret_access_key: str
    __output: str = ''
    __region: str = ''
    __mfa_serial: str = ''

    @property
    def profile_name(self) -> str:
        return self.__profile_name

    @property
    def output(self) -> str:
        return self.__output

    @property
    def region(self) -> str:
        return self.__region

    def check_mfa_serial(self) -> bool:
        print(self.__mfa_serial)
        if self.__mfa_serial:
            return True
        else:
            return False

    def get_sts_credential(self, token_code) -> Dict[str, str]:
        sesson = Session(
            aws_access_key_id=self.__aws_access_key_id,
            aws_secret_access_key=self.__aws_secret_access_key,
        )
        client = sesson.client('sts')
        try:
            response = client.get_session_token(
                DurationSeconds=DURATION_SECONDS,
                SerialNumber=self.__mfa_serial,
                TokenCode=token_code,
            )
        except botocore.exceptions.ClientError:
            print('STSからのクレデンシャルの取得に失敗しました')
            print('プロファイルの情報が正しいか、入力したコードが正しいことを確認して下さい')
            exit(1)
        return {
            'aws_access_key_id': response['Credentials']['AccessKeyId'],
            'aws_secret_access_key': response['Credentials']['SecretAccessKey'],
        }

    def export_string(self) -> str:
        profile_str: str = ''
        profile_str += '[{}]\n'.format(self.__profile_name)
        profile_str += 'aws_access_key_id={}\n'.format(self.__aws_access_key_id)
        profile_str += 'aws_secret_access_key={}\n'.format(self.__aws_secret_access_key)
        if self.__output:
            profile_str += 'output={}\n'.format(self.__output)
        if self.__region:
            profile_str += 'region={}\n'.format(self.__region)
        if self.__mfa_serial:
            profile_str += 'mfa_serial={}\n'.format(self.__mfa_serial)
        return profile_str

    def update_credential(self, aws_access_key_id, aws_secret_access_key) -> None:
        self.__aws_access_key_id = aws_access_key_id
        self.__aws_secret_access_key = aws_secret_access_key


def convert_credential_dict(credential_dict) -> List[str]:
    required_attribute_list = [
        'profile_name',
        'aws_access_key_id',
        'aws_secret_access_key',
        'output',
        'region',
        'mfa_serial',
    ]
    converted_attribute_list: List[str] = []
    for attribute in required_attribute_list:
        if attribute in credential_dict:
            converted_attribute_list.append(credential_dict[attribute])
        else:
            converted_attribute_list.append('')
    return converted_attribute_list



"""AWS CLIをMFA付きで実行するため、MFA検証済みのアクセスキーを設定するためのスクリプト"""
from typing import Dict, List
from pathlib import Path
from aws_mfa_util.credential import AwsCredential, convert_credential_dict
from aws_mfa_util.util import make_credential_str_list, convert_aws_credential

AWS_CREDENTIAL_PATH = str(Path.home() / '.aws/credentials')

try:
    from PyInquirer import prompt
except ImportError:
    print('PyInquirerがローカル環境にインストールされていません')
    exit(1)


def main():
    profile_list: List[str] = make_credential_str_list(AWS_CREDENTIAL_PATH)
    aws_credential_dict: Dict[str, AwsCredential] = convert_aws_credential(profile_list)

    # ========================================
    # プロファイルを新規作成するか、更新するかを質問する
    # ========================================
    action_questions = [
        {
            'type': 'list',
            'name': 'select_action',
            'message': '希望する処理を選択して下さい',
            'choices': ['MFA用プロファイルを新規作成', 'MFA用プロファイルの認証情報を更新'],
        }
    ]
    action_answers = prompt(action_questions)

    if action_answers['select_action'] == 'MFA用プロファイルを新規作成':
        # if: 新規でMFAユーザーを作成する場合

        # ========================================
        # どのプロファイルからMFAのプロファイルを作成するかを質問する
        # ========================================
        profile_questions = [
            {
                'type': 'list',
                'name': 'profile_name',
                'message': 'MFA用に新規作成するプロファイルを選択して下さい',
                'choices': [
                    profile_name
                    for profile_name in aws_credential_dict
                    if '-mfa' not in profile_name
                ],
            }
        ]
        # profile_answer = prompt(profile_questions)

        target_aws_credential: AwsCredential = aws_credential_dict[
            profile_answer['profile_name']
        ]
        if not target_aws_credential.check_mfa_serial():
            print('MFAデバイスのシリアルが登録されていません')
            exit(1)

        code_questions = [
            {'type': 'input', 'name': 'code', 'message': 'ワンタイムコードを入力して下さい'}
        ]
        code_answers = prompt(code_questions)

        # STSから一時クレデンシャルを取得する
        session_credential = target_aws_credential.get_sts_credential(
            code_answers['code']
        )

        # 取得した一時クレデンシャルを用いて、新規のAwsCredentialクラスを作成する
        new_aws_mfa_credential: AwsCredential = AwsCredential(
            profile_answer['profile_name'] + '-mfa',
            session_credential['aws_access_key_id'],
            session_credential['aws_secret_access_key'],
            aws_credential_dict[profile_answer['profile_name']].output,
            aws_credential_dict[profile_answer['profile_name']].region,
            '',
        )
        aws_credential_dict[
            new_aws_mfa_credential.profile_name
        ] = new_aws_mfa_credential

        final_message: str = 'MFA用プロファイルの新規作成が完了しました。'

    elif action_answers['select_action'] == 'MFA用プロファイルの認証情報を更新':
        # if: 既存のMFAプロファイルの情報を更新する

        # ========================================
        # どのプロファイルのMFA情報を更新するかを確認する
        # ========================================
        mfa_profile_list = [
            profile_name
            for profile_name in aws_credential_dict
            if '-mfa' in profile_name
        ]
        if not mfa_profile_list:
            print('MFA用のプロファイルがありません')
            exit(1)
        profile_questions = [
            {
                'type': 'list',
                'name': 'profile_name',
                'message': '更新するMFAプロファイルを選択して下さい',
                'choices': mfa_profile_list,
            }
        ]
        profile_answer = prompt(profile_questions)

        profile_name = profile_answer['profile_name']
        base_profile_name: str = profile_name.rstrip('-mfa')
        code_questions = [
            {'type': 'input', 'name': 'code', 'message': 'ワンタイムコードを入力して下さい'}
        ]
        code_answers = prompt(code_questions)

        session_credential = aws_credential_dict[base_profile_name].get_sts_credential(
            code_answers['code']
        )
        aws_credential_dict[profile_name].update_credential(
            session_credential['aws_access_key_id'],
            session_credential['aws_secret_access_key'],
        )

        final_message = 'MFA用プロファイルの認証情報更新が完了しました。'

    # 最終的なプロファイル情報をファイルに書き込む
    with open(AWS_CREDENTIAL_PATH, mode='w') as fd:
        for credential in aws_credential_dict.values():
            fd.write(credential.export_string() + '\n')
    
    print(final_message)


if __name__ == '__main__':
    main()



from aws_mfa_util.util import make_credential_str_list, convert_aws_credential


def test_make_credential_str_list():
    print('hello')
    # with open('./sample_credentials', mode='r') as fd:
    #     print('OK')
    # tmp = make_credential_str_list('./sample_credentials')
    # print(tmp)
    assert True



import setuptools

with open('README.md', 'r') as fd:
    long_description = fd.read()

setuptools.setup(
    name='aws-mfa-util',
    version='0.0.7',
    author='hiroya akita',
    author_email='akky.develop@gmail.com',
    description='AWS CLIにMFA適用アカウントを作成・更新するためのツール',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=['boto3', 'PyInquirer'],
    url='https://github.com/papi-tokei/aws-mfa-util',
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts':[
            'aws-mfa-util=aws_mfa_util:main'
        ],
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
    ],
    python_requires='>=3.6',
)
